class InvalidModelError(Exception):
    def __init__(self, message):
        super().__init__(message)

class InvalidModelJobError(Exception):
    def __init__(self, message):
        super().__init__(message)


import dataclasses
from typing import ClassVar, List

from pyspark.sql import SparkSession

from curia.infra.task import TaskDefinition, TaskInputDefinition
from curia.infra.types import TaskType, DatabricksTaskCodeSourceType


@dataclasses.dataclass
class DatabricksTaskDefinition(TaskDefinition):  # pylint: disable=too-many-instance-attributes
    """
    Task definition for a task executed through a databricks job
    """
    code_src_type: DatabricksTaskCodeSourceType = DatabricksTaskCodeSourceType.S3WHEEL
    code_src_cfg: dict = dataclasses.field(default_factory=dict)
    min_workers: int = 2
    max_workers: int = 4
    task_type: ClassVar[TaskType] = TaskType.DATABRICKS

    # pylint: disable=arguments-differ
    def run(self,
            task_execution_id: str,
            api_token: str,
            api_endpoint: str,
            spark: SparkSession,
            dbutils: 'DBUtils') -> None:
        """
        Run the analytics task flow definition
        :param task_execution_id: The task execution ID
        :param api_token: The API key to use to retrieve the task inputs from the Curia API
        :param api_endpoint: The API endpoint to use to retrieve the task inputs from the Curia API
        :return: The result
        """
        assert self._function is not None, "TaskDefinition must decorate a function"
        resolved_args = self.resolve_arguments(task_execution_id, api_token, api_endpoint)
        results = self._function(**resolved_args, spark=spark, dbutils=dbutils)
        self.upload_results(task_execution_id, api_token, api_endpoint, results)

    def build_workflow_data_block(self, inputs: List[TaskInputDefinition], task_specific_context: dict):
        if "databricks_job_id" not in task_specific_context:
            raise ValueError("Databricks job ID must be provided to DatabricksTaskDefinition")
        databricks_job_id = task_specific_context["databricks_job_id"]
        return {
            "notebook_params": {
                "code_version": {
                    "required": True,
                    "descr": "Code version to use for the task"
                },
            },
            "job_id": databricks_job_id,
            **{
                task_input.name: {
                    "required": not task_input.optional,
                    "descr": task_input.description
                } for task_input in inputs
            }
        }

    def build_task_inputs(self, inputs: List[TaskInputDefinition], task_specific_context: dict):
        if "databricks_job_id" not in task_specific_context:
            raise ValueError("Databricks job ID must be provided to DatabricksTaskDefinition")
        databricks_job_id = task_specific_context["databricks_job_id"]
        return {
            "notebook_params": {
                "code_version": {
                    "required": True,
                    "descr": "Code version to use for the task"
                },
            },
            "job_id": databricks_job_id,
            **{
                task_input.name: {
                    "required": not task_input.optional,
                    "descr": task_input.description
                } for task_input in inputs
            }
        }



from email.message import EmailMessage
import dataclasses
import functools
import itertools
import json
import pathlib
import time
import traceback
import urllib.parse
from http.server import HTTPServer, BaseHTTPRequestHandler
from multiprocessing.context import Process
from typing import List, Union, Any

import multipart
import requests


def parse_content_type_header(header):
    """
    Parse a content-type header into a tuple of the content type and a dictionary of parameters.
    :param header: the content-type header to parse
    :return: a tuple of the content type and a dictionary of parameters

    See https://docs.python.org/3/library/cgi.html#functions-in-cgi-module for why we are using EmailMessage here
    instead of the cgi module.
    """
    msg = EmailMessage()
    msg['content-type'] = header
    return msg.get_content_type(), dict(msg['content-type'].params)


def parse_multipart(rfile, post_dict):
    """
    Parses a multipart/form-data request body into a dictionary of lists of strings, replicating the behavior of
    cgi.parse_multipart.
    :param rfile: the file-like object to read the request body from
    :param post_dict: contains additional parameters from the content-type header
    :return: the populated dictionary
    """
    msg = EmailMessage()
    msg.set_content(rfile.read(), **post_dict)
    return {**msg.get_params()}


class ServerFailedToStartException(Exception):
    pass


def using_test_server(port=3000, response_map=None, log_dir="temp/", response_map_metadata: Any = None):
    def wrap(fn):
        @functools.wraps(fn)
        def wrapper(*args, **kwargs):
            server_process = Process(
                target=run_test_server,
                kwargs={
                    "port": port,
                    "response_map": response_map,
                    "log_dir": log_dir,
                    "response_map_metadata": response_map_metadata
                },
                daemon=False
            )
            server_process.start()
            time.sleep(2)
            failed_to_start = False
            r = None
            try:
                r = requests.get(f'http://localhost:{port}/healthcheck', timeout=10)
            except requests.exceptions.ConnectionError:
                failed_to_start = True
            failed_to_start = failed_to_start or r.status_code != 200
            if failed_to_start:
                if server_process.is_alive():
                    server_process.terminate()
                    server_process.join(timeout=5)
                    if server_process.is_alive():
                        print('Killing server process...')
                        server_process.kill()
                server_logs = (pathlib.Path(log_dir) / "test_server_logs.log").read_text()
                raise ServerFailedToStartException(f"Server failed to start! {server_logs}")
            try:
                res = fn(*args, **kwargs)
            finally:
                print('Terminating server process...')
                server_process.terminate()
                server_process.join(timeout=5)
                if server_process.is_alive():
                    print('Killing server process...')
                    server_process.kill()
                print('Server process terminated.')
            return res

        return wrapper

    return wrap


def run_test_server(port=3000, response_map=None, log_dir="temp/", response_map_metadata: Any = None):
    server = TestServer(
        port=port,
        response_map=response_map,
        log_dir=log_dir,
        response_map_metadata=response_map_metadata
    )
    server.serve_forever(50)


@dataclasses.dataclass
class ParseAndExtractDataSummary:
    method: str
    path: str
    url_qs: dict
    content_qs: dict
    content_type: str


def default_encoder(obj):
    try:
        json.dumps(obj)
        return obj
    except TypeError:
        return str(obj)


class Exception404(Exception):
    pass


class TestServer(HTTPServer):
    def __init__(self,
                 port=3000,
                 shutdown_poll_freq=50,
                 response_map=None,
                 log_dir="temp/",
                 response_map_metadata: Any = None):
        if response_map is None:
            self.response_map = {}
        else:
            self.response_map = response_map
        print(f"Starting server on port {port}")
        self.started = False
        self.process = None
        self.log_dir = log_dir
        self.log_list = []
        self.shutdown_poll_freq = shutdown_poll_freq
        self.response_map_metadata = response_map_metadata
        super().__init__(('', port), TestServer.RequestHandler)

    @dataclasses.dataclass
    class ResponseConfig:
        accepted_methods: Union[List[str], str]
        content_type: str
        content: bytes

        def accepts_method(self, method):
            if self.accepted_methods == 'all':
                return True
            return method.lower() in [accepted_method.lower() for accepted_method in self.accepted_methods]

        def add_data(self, handler):
            handler.send_header("Content-type", self.content_type)
            handler.end_headers()
            handler.wfile.write(self.content)

    class RequestHandler(BaseHTTPRequestHandler):
        server: 'TestServer'

        def do_GET(self):
            self.map_to_response("GET")

        def do_POST(self):
            self.map_to_response("POST")

        def do_PATCH(self):
            self.map_to_response("PATCH")

        def do_PUT(self):
            self.map_to_response("PUT")

        def do_DELETE(self):
            self.map_to_response("DELETE")

        def parse_and_extract(self, method: str):
            parsed_url = urllib.parse.urlparse(f"http://localhost:{self.server.server_port}{self.path}")
            url_qs = urllib.parse.parse_qs(parsed_url.query)
            if self.headers.get("content-type") is not None:
                content_type, post_dict = parse_content_type_header(self.headers.get('content-type'))
                if content_type == 'multipart/form-data':
                    length = int(self.headers.get('content-length'))
                    post_dict['CONTENT-LENGTH'] = length
                    parser = multipart.MultipartParser(self.rfile, post_dict['boundary'], content_length=length)
                    content_qs = {}
                    for part in parser:
                        if part.name is not None:
                            content_qs[part.name] = part.value
                    post_dict['boundary'] = post_dict['boundary'].encode("utf-8")

                elif content_type in ['application/x-www-form-urlencoded', 'application/json']:
                    length = int(self.headers.get('content-length', 0))
                    read_data = self.rfile.read(length).decode('utf-8')
                    if read_data == "":
                        content_qs = {}
                    elif content_type == 'application/x-www-form-urlencoded':
                        content_qs = urllib.parse.parse_qs(self.rfile.read(length), keep_blank_values=True)
                    else:
                        content_qs = json.loads(read_data)
                else:
                    content_qs = {}
            else:
                content_qs = {}
                content_type = None
            return ParseAndExtractDataSummary(
                method=method,
                content_qs=content_qs,
                url_qs=url_qs,
                content_type=content_type,
                path=parsed_url.path
            )

        def map_to_response_auto(self, method):
            if method == "GET" and self.path == "/debug-logs":
                self.send_response(200)
                self.add_debug_logs()
                return True
            if method == "GET" and self.path == "/healthcheck":
                self.send_response(200)
                self.add_healthcheck()
                return True
            if method == "GET" and self.path == "/teapot":
                self.send_response(418)
                self.add_teapot()
                return True
            return False

        def use_response_map_callable(self, parse_extract_summary: ParseAndExtractDataSummary):
            try:
                response_map_kwargs = {
                    'metadata': self.server.response_map_metadata
                } if self.server.response_map_metadata is not None else {}
                response_value = self.server.response_map(
                    parse_extract_summary,
                    **response_map_kwargs
                )
                self.send_response(200)
                self.infer_type_and_add_data(response_value)
                return
            except Exception404 as e:
                self.send_response(404)
                self.add_404(e)
                return
            except Exception as e:
                self.send_response(500)
                self.add_500(e, traceback.format_exc())
                return

        def use_response_map_map(self, method: str, parse_extract_summary: ParseAndExtractDataSummary):
            if callable(self.server.response_map[parse_extract_summary.path]):
                response_value = self.server.response_map[parse_extract_summary.path](parse_extract_summary)
            else:
                response_value = self.server.response_map[parse_extract_summary.path]
            if isinstance(response_value, TestServer.ResponseConfig):
                if response_value.accepts_method(method):
                    self.send_response(200)
                else:
                    self.send_response(405)
                    self.add_405()
                    return
                response_value.add_data(self)
            else:
                self.send_response(200)
                self.infer_type_and_add_data(response_value)
                return

        def map_to_response(self, method):
            try:
                if self.map_to_response_auto(method):
                    return
                parse_extract_summary = self.parse_and_extract(method)
                self.server.log(method, self.path, parse_extract_summary)
                if callable(self.server.response_map):
                    self.use_response_map_callable(parse_extract_summary)
                    return
                if isinstance(self.server.response_map, dict) and parse_extract_summary.path in self.server.response_map:
                    self.use_response_map_map(method, parse_extract_summary)
                    return
                self.send_response(404)
                self.add_404()
                return
            except Exception as e:
                self.send_response(500)
                self.add_500(e, traceback.format_exc())
                return

        def infer_type_and_add_data(self, response_value):
            if isinstance(response_value, dict):
                self.send_header("Content-type", "application/json")
                self.end_headers()
                data = json.dumps(response_value, default=default_encoder).encode('utf-8')
                self.wfile.write(data)
            elif isinstance(response_value, list):
                self.send_header("Content-type", "application/json")
                self.end_headers()
                data = json.dumps(response_value, default=default_encoder).encode('utf-8')
                self.wfile.write(data)
            elif hasattr(response_value, "to_dict"):
                self.send_header("Content-type", "application/json")
                self.end_headers()
                data = json.dumps(response_value.to_dict(), default=default_encoder).encode('utf-8')
                self.wfile.write(data)
            elif isinstance(response_value, str):
                self.send_header("Content-type", "text/html")
                self.end_headers()
                data = response_value.encode('utf-8')
                self.wfile.write(data)
            else:
                self.send_header("Content-type", "text/plain")
                self.end_headers()
                data = str(response_value).encode('utf-8')
                self.wfile.write(data)

        def add_debug_logs(self):
            results = list(itertools.starmap(
                lambda method, path, summary: {"method": method, "path": path, "summary": dataclasses.asdict(summary)},
                self.server.log_list
            ))
            self.send_header("Content-type", "application/json")
            self.end_headers()
            data = json.dumps(results, default=default_encoder).encode('utf-8')
            self.wfile.write(data)

        def add_404(self, e: Exception = None):
            self.send_header("Content-type", "application/json")
            self.end_headers()
            data = json.dumps({
                "error": "404 error path not configured",
                "exception": str(e) if e else None
            }, default=default_encoder).encode('utf-8')
            self.wfile.write(data)

        def add_500(self, e: Exception = None, tb: str = None):
            self.send_header("Content-type", "application/json")
            self.end_headers()
            data = json.dumps({"error": f"500 internal server error: {e} \n {tb}"}, default=default_encoder)\
                .encode('utf-8')
            self.wfile.write(data)

        def add_405(self):
            self.send_header("Content-type", "application/json")
            self.end_headers()
            data = json.dumps({"error": "405 error method not supported"}, default=default_encoder).encode('utf-8')
            self.wfile.write(data)

        def add_healthcheck(self):
            self.send_header("Content-type", "application/json")
            self.end_headers()
            data = json.dumps({"healthy": "true"}, default=default_encoder).encode('utf-8')
            self.wfile.write(data)

        def add_teapot(self):
            self.send_header("Content-type", "application/json")
            self.end_headers()
            data = json.dumps({"my-status": "I'm a teapot"}, default=default_encoder).encode('utf-8')
            self.wfile.write(data)

    def log(self, method, path, summary: ParseAndExtractDataSummary):
        self.log_list.append((method, path, summary))
        path = pathlib.Path(self.log_dir)
        path.mkdir(exist_ok=True, parents=True)
        with open(path / "test_server_logs.log", "a+", encoding='utf8') as file:
            file.write(f"{method} {path}: {json.dumps(dataclasses.asdict(summary), default=default_encoder)}\n")

    @staticmethod
    def debug_logs(port):
        """
        Accesses debug logs of a test server running on a given port
        """
        logs = requests.get(f"http://localhost:{port}/debug-logs", timeout=10)
        return json.loads(logs.content)



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.11.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import re  # noqa: F401

# python 2 and python 3 compatibility library
import six

from curia.api.swagger_client.api_client import ApiClient


class DataTablesApi(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    Ref: https://github.com/swagger-api/swagger-codegen
    """

    def __init__(self, api_client=None):
        if api_client is None:
            api_client = ApiClient()
        self.api_client = api_client

    def get_many_base_data_table_controller_data_table(self, **kwargs):  # noqa: E501
        """Retrieve multiple DataTables  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_many_base_data_table_controller_data_table(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param list[str] fields: Selects resource fields. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#select\" target=\"_blank\">Docs</a>
        :param str s: Adds search condition. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#search\" target=\"_blank\">Docs</a>
        :param list[str] filter: Adds filter condition. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#filter\" target=\"_blank\">Docs</a>
        :param list[str] _or: Adds OR condition. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#or\" target=\"_blank\">Docs</a>
        :param list[str] sort: Adds sort by field. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#sort\" target=\"_blank\">Docs</a>
        :param list[str] join: Adds relational resources. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#join\" target=\"_blank\">Docs</a>
        :param int limit: Limit amount of resources. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#limit\" target=\"_blank\">Docs</a>
        :param int offset: Offset amount of resources. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#offset\" target=\"_blank\">Docs</a>
        :param int page: Page portion of resources. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#page\" target=\"_blank\">Docs</a>
        :param int cache: Reset cache (if was enabled). <a href=\"https://github.com/nestjsx/crud/wiki/Requests#cache\" target=\"_blank\">Docs</a>
        :return: GetManyDataTableResponseDto
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_many_base_data_table_controller_data_table_with_http_info(**kwargs)  # noqa: E501
        else:
            (data) = self.get_many_base_data_table_controller_data_table_with_http_info(**kwargs)  # noqa: E501
            return data

    def get_many_base_data_table_controller_data_table_with_http_info(self, **kwargs):  # noqa: E501
        """Retrieve multiple DataTables  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_many_base_data_table_controller_data_table_with_http_info(async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param list[str] fields: Selects resource fields. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#select\" target=\"_blank\">Docs</a>
        :param str s: Adds search condition. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#search\" target=\"_blank\">Docs</a>
        :param list[str] filter: Adds filter condition. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#filter\" target=\"_blank\">Docs</a>
        :param list[str] _or: Adds OR condition. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#or\" target=\"_blank\">Docs</a>
        :param list[str] sort: Adds sort by field. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#sort\" target=\"_blank\">Docs</a>
        :param list[str] join: Adds relational resources. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#join\" target=\"_blank\">Docs</a>
        :param int limit: Limit amount of resources. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#limit\" target=\"_blank\">Docs</a>
        :param int offset: Offset amount of resources. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#offset\" target=\"_blank\">Docs</a>
        :param int page: Page portion of resources. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#page\" target=\"_blank\">Docs</a>
        :param int cache: Reset cache (if was enabled). <a href=\"https://github.com/nestjsx/crud/wiki/Requests#cache\" target=\"_blank\">Docs</a>
        :return: GetManyDataTableResponseDto
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['fields', 's', 'filter', '_or', 'sort', 'join', 'limit', 'offset', 'page', 'cache']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_many_base_data_table_controller_data_table" % key
                )
            params[key] = val
        del params['kwargs']

        collection_formats = {}

        path_params = {}

        query_params = []
        if 'fields' in params:
            query_params.append(('fields', params['fields']))  # noqa: E501
            collection_formats['fields'] = 'csv'  # noqa: E501
        if 's' in params:
            query_params.append(('s', params['s']))  # noqa: E501
        if 'filter' in params:
            query_params.append(('filter', params['filter']))  # noqa: E501
            collection_formats['filter'] = 'multi'  # noqa: E501
        if '_or' in params:
            query_params.append(('or', params['_or']))  # noqa: E501
            collection_formats['or'] = 'multi'  # noqa: E501
        if 'sort' in params:
            query_params.append(('sort', params['sort']))  # noqa: E501
            collection_formats['sort'] = 'multi'  # noqa: E501
        if 'join' in params:
            query_params.append(('join', params['join']))  # noqa: E501
            collection_formats['join'] = 'multi'  # noqa: E501
        if 'limit' in params:
            query_params.append(('limit', params['limit']))  # noqa: E501
        if 'offset' in params:
            query_params.append(('offset', params['offset']))  # noqa: E501
        if 'page' in params:
            query_params.append(('page', params['page']))  # noqa: E501
        if 'cache' in params:
            query_params.append(('cache', params['cache']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['api_key']  # noqa: E501

        return self.api_client.call_api(
            '/data-tables', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='GetManyDataTableResponseDto',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)

    def get_one_base_data_table_controller_data_table(self, id, **kwargs):  # noqa: E501
        """Retrieve a single DataTable  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_one_base_data_table_controller_data_table(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: (required)
        :param list[str] fields: Selects resource fields. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#select\" target=\"_blank\">Docs</a>
        :param list[str] join: Adds relational resources. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#join\" target=\"_blank\">Docs</a>
        :param int cache: Reset cache (if was enabled). <a href=\"https://github.com/nestjsx/crud/wiki/Requests#cache\" target=\"_blank\">Docs</a>
        :return: DataTableResponseDto
                 If the method is called asynchronously,
                 returns the request thread.
        """
        kwargs['_return_http_data_only'] = True
        if kwargs.get('async_req'):
            return self.get_one_base_data_table_controller_data_table_with_http_info(id, **kwargs)  # noqa: E501
        else:
            (data) = self.get_one_base_data_table_controller_data_table_with_http_info(id, **kwargs)  # noqa: E501
            return data

    def get_one_base_data_table_controller_data_table_with_http_info(self, id, **kwargs):  # noqa: E501
        """Retrieve a single DataTable  # noqa: E501

        This method makes a synchronous HTTP request by default. To make an
        asynchronous HTTP request, please pass async_req=True
        >>> thread = api.get_one_base_data_table_controller_data_table_with_http_info(id, async_req=True)
        >>> result = thread.get()

        :param async_req bool
        :param str id: (required)
        :param list[str] fields: Selects resource fields. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#select\" target=\"_blank\">Docs</a>
        :param list[str] join: Adds relational resources. <a href=\"https://github.com/nestjsx/crud/wiki/Requests#join\" target=\"_blank\">Docs</a>
        :param int cache: Reset cache (if was enabled). <a href=\"https://github.com/nestjsx/crud/wiki/Requests#cache\" target=\"_blank\">Docs</a>
        :return: DataTableResponseDto
                 If the method is called asynchronously,
                 returns the request thread.
        """

        all_params = ['id', 'fields', 'join', 'cache']  # noqa: E501
        all_params.append('async_req')
        all_params.append('_return_http_data_only')
        all_params.append('_preload_content')
        all_params.append('_request_timeout')

        params = locals()
        for key, val in six.iteritems(params['kwargs']):
            if key not in all_params:
                raise TypeError(
                    "Got an unexpected keyword argument '%s'"
                    " to method get_one_base_data_table_controller_data_table" % key
                )
            params[key] = val
        del params['kwargs']
        # verify the required parameter 'id' is set
        if ('id' not in params or
                params['id'] is None):
            raise ValueError("Missing the required parameter `id` when calling `get_one_base_data_table_controller_data_table`")  # noqa: E501

        collection_formats = {}

        path_params = {}
        if 'id' in params:
            path_params['id'] = params['id']  # noqa: E501

        query_params = []
        if 'fields' in params:
            query_params.append(('fields', params['fields']))  # noqa: E501
            collection_formats['fields'] = 'csv'  # noqa: E501
        if 'join' in params:
            query_params.append(('join', params['join']))  # noqa: E501
            collection_formats['join'] = 'multi'  # noqa: E501
        if 'cache' in params:
            query_params.append(('cache', params['cache']))  # noqa: E501

        header_params = {}

        form_params = []
        local_var_files = {}

        body_params = None
        # HTTP header `Accept`
        header_params['Accept'] = self.api_client.select_header_accept(
            ['application/json'])  # noqa: E501

        # Authentication setting
        auth_settings = ['api_key']  # noqa: E501

        return self.api_client.call_api(
            '/data-tables/{id}', 'GET',
            path_params,
            query_params,
            header_params,
            body=body_params,
            post_params=form_params,
            files=local_var_files,
            response_type='DataTableResponseDto',  # noqa: E501
            auth_settings=auth_settings,
            async_req=params.get('async_req'),
            _return_http_data_only=params.get('_return_http_data_only'),
            _preload_content=params.get('_preload_content', True),
            _request_timeout=params.get('_request_timeout'),
            collection_formats=collection_formats)



import functools
import json
from typing import Dict

import pandas as pd
from moto import mock_s3

from curia.utils.s3 import s3_uri_to_bucket_key, AWSIoContext


def _create_seed_s3_data(seed_s3_data: Dict[str, dict]):
    if not isinstance(seed_s3_data, dict):
        raise TypeError("seed_s3_data should be a dict!")
    s3_client = AWSIoContext.get_boto3_client()
    created_buckets = set()
    for s3_uri, object_data in seed_s3_data.items():
        bucket, key = s3_uri_to_bucket_key(s3_uri)
        if bucket not in created_buckets:
            s3_client.create_bucket(
                Bucket=bucket
            )
            created_buckets.add(bucket)

        if 'Body' in object_data:
            # sanitize the body intelligently
            if isinstance(object_data['Body'], str):
                object_data['Body'] = object_data['Body'].encode()
            elif isinstance(object_data['Body'], (dict, list)):
                object_data['Body'] = json.dumps(object_data['Body']).encode()
            elif isinstance(object_data['Body'], pd.DataFrame):
                object_data['Body'] = object_data['Body'].to_parquet()

        s3_client.put_object(
            Bucket=bucket,
            Key=key,
            **object_data
        )


def curia_mock_s3(seed_s3_data: Dict[str, dict]):
    def decorator(fn):
        @functools.wraps(fn)
        def wrapper(*args, **kwargs):
            _create_seed_s3_data(seed_s3_data)
            return fn(*args, **kwargs)
        wrapper = mock_s3(wrapper)
        return wrapper
    return decorator


# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 2.14.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

import curia.api.curia.api.curia.api.curia.api.swagger_client
from curia.api.curia.api.curia.api.curia.api.swagger_client.api.user_management_api import UserManagementApi  # noqa: E501
from curia.api.curia.api.curia.api.curia.api.swagger_client.rest import ApiException


class TestUserManagementApi(unittest.TestCase):
    """UserManagementApi unit test stubs"""

    def setUp(self):
        self.api = UserManagementApi()  # noqa: E501

    def tearDown(self):
        pass

    def test_user_management_controller_assign_user_roles(self):
        """Test case for user_management_controller_assign_user_roles

        Assign user roles  # noqa: E501
        """
        pass

    def test_user_management_controller_cancel_user_invite(self):
        """Test case for user_management_controller_cancel_user_invite

        Cancel user invitation  # noqa: E501
        """
        pass

    def test_user_management_controller_get_my_user(self):
        """Test case for user_management_controller_get_my_user

        Retrieve your user  # noqa: E501
        """
        pass

    def test_user_management_controller_get_my_user_api_key(self):
        """Test case for user_management_controller_get_my_user_api_key

        Retrieve your api-key  # noqa: E501
        """
        pass

    def test_user_management_controller_get_my_user_organizations(self):
        """Test case for user_management_controller_get_my_user_organizations

        Retrieve user organizations  # noqa: E501
        """
        pass

    def test_user_management_controller_get_roles(self):
        """Test case for user_management_controller_get_roles

        Retrieves user roles  # noqa: E501
        """
        pass

    def test_user_management_controller_get_user(self):
        """Test case for user_management_controller_get_user

        Retrieve one user  # noqa: E501
        """
        pass

    def test_user_management_controller_get_users(self):
        """Test case for user_management_controller_get_users

        Retrieve many users  # noqa: E501
        """
        pass

    def test_user_management_controller_invite_user(self):
        """Test case for user_management_controller_invite_user

        Invite a User  # noqa: E501
        """
        pass

    def test_user_management_controller_refresh_api_key(self):
        """Test case for user_management_controller_refresh_api_key

        Refresh api key for userid  # noqa: E501
        """
        pass

    def test_user_management_controller_refresh_my_api_key(self):
        """Test case for user_management_controller_refresh_my_api_key

        Refresh user api key  # noqa: E501
        """
        pass

    def test_user_management_controller_remove_user(self):
        """Test case for user_management_controller_remove_user

        Delete one User  # noqa: E501
        """
        pass

    def test_user_management_controller_remove_user_roles(self):
        """Test case for user_management_controller_remove_user_roles

        Remove user roles  # noqa: E501
        """
        pass

    def test_user_management_controller_request_password_reset(self):
        """Test case for user_management_controller_request_password_reset

        Request user password reset  # noqa: E501
        """
        pass

    def test_user_management_controller_resend_user_invite(self):
        """Test case for user_management_controller_resend_user_invite

        Resend user invitation  # noqa: E501
        """
        pass

    def test_user_management_controller_update_my_user(self):
        """Test case for user_management_controller_update_my_user

        Update my User  # noqa: E501
        """
        pass

    def test_user_management_controller_update_user(self):
        """Test case for user_management_controller_update_user

        Update one User  # noqa: E501
        """
        pass

    def test_user_management_controller_update_user_metadata(self):
        """Test case for user_management_controller_update_user_metadata

        Update one user metadata  # noqa: E501
        """
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.2.0-release-3.x.5
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

from __future__ import absolute_import

import unittest

import curia.api.curia.api.curia.api.curia.api.swagger_client
from curia.api.curia.api.curia.api.curia.api.swagger_client.models.model_job_joined_model_population_joined_outcome_definition_response_dto import ModelJobJoinedModelPopulationJoinedOutcomeDefinitionResponseDto  # noqa: E501
from curia.api.curia.api.curia.api.curia.api.swagger_client.rest import ApiException


class TestModelJobJoinedModelPopulationJoinedOutcomeDefinitionResponseDto(unittest.TestCase):
    """ModelJobJoinedModelPopulationJoinedOutcomeDefinitionResponseDto unit test stubs"""

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def testModelJobJoinedModelPopulationJoinedOutcomeDefinitionResponseDto(self):
        """Test ModelJobJoinedModelPopulationJoinedOutcomeDefinitionResponseDto"""
        # FIXME: construct object with mandatory attributes with example values
        # model = curia.api.curia.api.curia.api.swagger_client.models.model_job_joined_model_population_joined_outcome_definition_response_dto.ModelJobJoinedModelPopulationJoinedOutcomeDefinitionResponseDto()  # noqa: E501
        pass


if __name__ == '__main__':
    unittest.main()



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.11.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class UpdateUserDto(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'email': 'str',
        'phone_number': 'str',
        'given_name': 'str',
        'family_name': 'str',
        'name': 'str',
        'nickname': 'str',
        'picture': 'str',
        'password': 'str',
        'username': 'str',
        'verify_email': 'bool',
        'blocked': 'bool'
    }

    attribute_map = {
        'email': 'email',
        'phone_number': 'phone_number',
        'given_name': 'given_name',
        'family_name': 'family_name',
        'name': 'name',
        'nickname': 'nickname',
        'picture': 'picture',
        'password': 'password',
        'username': 'username',
        'verify_email': 'verify_email',
        'blocked': 'blocked'
    }

    def __init__(self, email=None, phone_number=None, given_name=None, family_name=None, name=None, nickname=None, picture=None, password=None, username=None, verify_email=None, blocked=None):  # noqa: E501
        """UpdateUserDto - a model defined in Swagger"""  # noqa: E501
        self._email = None
        self._phone_number = None
        self._given_name = None
        self._family_name = None
        self._name = None
        self._nickname = None
        self._picture = None
        self._password = None
        self._username = None
        self._verify_email = None
        self._blocked = None
        self.discriminator = None
        if email is not None:
            self.email = email
        if phone_number is not None:
            self.phone_number = phone_number
        if given_name is not None:
            self.given_name = given_name
        if family_name is not None:
            self.family_name = family_name
        if name is not None:
            self.name = name
        if nickname is not None:
            self.nickname = nickname
        if picture is not None:
            self.picture = picture
        if password is not None:
            self.password = password
        if username is not None:
            self.username = username
        if verify_email is not None:
            self.verify_email = verify_email
        if blocked is not None:
            self.blocked = blocked

    @property
    def email(self):
        """Gets the email of this UpdateUserDto.  # noqa: E501


        :return: The email of this UpdateUserDto.  # noqa: E501
        :rtype: str
        """
        return self._email

    @email.setter
    def email(self, email):
        """Sets the email of this UpdateUserDto.


        :param email: The email of this UpdateUserDto.  # noqa: E501
        :type: str
        """

        self._email = email

    @property
    def phone_number(self):
        """Gets the phone_number of this UpdateUserDto.  # noqa: E501


        :return: The phone_number of this UpdateUserDto.  # noqa: E501
        :rtype: str
        """
        return self._phone_number

    @phone_number.setter
    def phone_number(self, phone_number):
        """Sets the phone_number of this UpdateUserDto.


        :param phone_number: The phone_number of this UpdateUserDto.  # noqa: E501
        :type: str
        """

        self._phone_number = phone_number

    @property
    def given_name(self):
        """Gets the given_name of this UpdateUserDto.  # noqa: E501


        :return: The given_name of this UpdateUserDto.  # noqa: E501
        :rtype: str
        """
        return self._given_name

    @given_name.setter
    def given_name(self, given_name):
        """Sets the given_name of this UpdateUserDto.


        :param given_name: The given_name of this UpdateUserDto.  # noqa: E501
        :type: str
        """

        self._given_name = given_name

    @property
    def family_name(self):
        """Gets the family_name of this UpdateUserDto.  # noqa: E501


        :return: The family_name of this UpdateUserDto.  # noqa: E501
        :rtype: str
        """
        return self._family_name

    @family_name.setter
    def family_name(self, family_name):
        """Sets the family_name of this UpdateUserDto.


        :param family_name: The family_name of this UpdateUserDto.  # noqa: E501
        :type: str
        """

        self._family_name = family_name

    @property
    def name(self):
        """Gets the name of this UpdateUserDto.  # noqa: E501


        :return: The name of this UpdateUserDto.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this UpdateUserDto.


        :param name: The name of this UpdateUserDto.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def nickname(self):
        """Gets the nickname of this UpdateUserDto.  # noqa: E501


        :return: The nickname of this UpdateUserDto.  # noqa: E501
        :rtype: str
        """
        return self._nickname

    @nickname.setter
    def nickname(self, nickname):
        """Sets the nickname of this UpdateUserDto.


        :param nickname: The nickname of this UpdateUserDto.  # noqa: E501
        :type: str
        """

        self._nickname = nickname

    @property
    def picture(self):
        """Gets the picture of this UpdateUserDto.  # noqa: E501


        :return: The picture of this UpdateUserDto.  # noqa: E501
        :rtype: str
        """
        return self._picture

    @picture.setter
    def picture(self, picture):
        """Sets the picture of this UpdateUserDto.


        :param picture: The picture of this UpdateUserDto.  # noqa: E501
        :type: str
        """

        self._picture = picture

    @property
    def password(self):
        """Gets the password of this UpdateUserDto.  # noqa: E501


        :return: The password of this UpdateUserDto.  # noqa: E501
        :rtype: str
        """
        return self._password

    @password.setter
    def password(self, password):
        """Sets the password of this UpdateUserDto.


        :param password: The password of this UpdateUserDto.  # noqa: E501
        :type: str
        """

        self._password = password

    @property
    def username(self):
        """Gets the username of this UpdateUserDto.  # noqa: E501


        :return: The username of this UpdateUserDto.  # noqa: E501
        :rtype: str
        """
        return self._username

    @username.setter
    def username(self, username):
        """Sets the username of this UpdateUserDto.


        :param username: The username of this UpdateUserDto.  # noqa: E501
        :type: str
        """

        self._username = username

    @property
    def verify_email(self):
        """Gets the verify_email of this UpdateUserDto.  # noqa: E501


        :return: The verify_email of this UpdateUserDto.  # noqa: E501
        :rtype: bool
        """
        return self._verify_email

    @verify_email.setter
    def verify_email(self, verify_email):
        """Sets the verify_email of this UpdateUserDto.


        :param verify_email: The verify_email of this UpdateUserDto.  # noqa: E501
        :type: bool
        """

        self._verify_email = verify_email

    @property
    def blocked(self):
        """Gets the blocked of this UpdateUserDto.  # noqa: E501


        :return: The blocked of this UpdateUserDto.  # noqa: E501
        :rtype: bool
        """
        return self._blocked

    @blocked.setter
    def blocked(self, blocked):
        """Sets the blocked of this UpdateUserDto.


        :param blocked: The blocked of this UpdateUserDto.  # noqa: E501
        :type: bool
        """

        self._blocked = blocked

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(UpdateUserDto, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, UpdateUserDto):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.11.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class ModelJobJoinedModelJobStatusResponseDto(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'model_job_id': 'str',
        'project_id': 'str',
        'source': 'str',
        'step': 'str',
        'message': 'str',
        'type': 'str',
        'order': 'float',
        'created_at': 'datetime',
        'updated_at': 'datetime',
        'archived_at': 'datetime',
        'last_updated_by': 'str',
        'created_by': 'str',
        'progress': 'float',
        'metadata': 'object',
        'version': 'float'
    }

    attribute_map = {
        'id': 'id',
        'model_job_id': 'modelJobId',
        'project_id': 'projectId',
        'source': 'source',
        'step': 'step',
        'message': 'message',
        'type': 'type',
        'order': 'order',
        'created_at': 'createdAt',
        'updated_at': 'updatedAt',
        'archived_at': 'archivedAt',
        'last_updated_by': 'lastUpdatedBy',
        'created_by': 'createdBy',
        'progress': 'progress',
        'metadata': 'metadata',
        'version': 'version'
    }

    def __init__(self, id=None, model_job_id=None, project_id=None, source=None, step=None, message=None, type=None, order=None, created_at=None, updated_at=None, archived_at=None, last_updated_by=None, created_by=None, progress=None, metadata=None, version=None):  # noqa: E501
        """ModelJobJoinedModelJobStatusResponseDto - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._model_job_id = None
        self._project_id = None
        self._source = None
        self._step = None
        self._message = None
        self._type = None
        self._order = None
        self._created_at = None
        self._updated_at = None
        self._archived_at = None
        self._last_updated_by = None
        self._created_by = None
        self._progress = None
        self._metadata = None
        self._version = None
        self.discriminator = None
        if id is not None:
            self.id = id
        self.model_job_id = model_job_id
        self.project_id = project_id
        if source is not None:
            self.source = source
        self.step = step
        if message is not None:
            self.message = message
        if type is not None:
            self.type = type
        if order is not None:
            self.order = order
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at
        if archived_at is not None:
            self.archived_at = archived_at
        if last_updated_by is not None:
            self.last_updated_by = last_updated_by
        if created_by is not None:
            self.created_by = created_by
        if progress is not None:
            self.progress = progress
        if metadata is not None:
            self.metadata = metadata
        if version is not None:
            self.version = version

    @property
    def id(self):
        """Gets the id of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501


        :return: The id of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this ModelJobJoinedModelJobStatusResponseDto.


        :param id: The id of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def model_job_id(self):
        """Gets the model_job_id of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501


        :return: The model_job_id of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._model_job_id

    @model_job_id.setter
    def model_job_id(self, model_job_id):
        """Sets the model_job_id of this ModelJobJoinedModelJobStatusResponseDto.


        :param model_job_id: The model_job_id of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :type: str
        """
        if model_job_id is None:
            raise ValueError("Invalid value for `model_job_id`, must not be `None`")  # noqa: E501

        self._model_job_id = model_job_id

    @property
    def project_id(self):
        """Gets the project_id of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501


        :return: The project_id of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._project_id

    @project_id.setter
    def project_id(self, project_id):
        """Sets the project_id of this ModelJobJoinedModelJobStatusResponseDto.


        :param project_id: The project_id of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :type: str
        """
        if project_id is None:
            raise ValueError("Invalid value for `project_id`, must not be `None`")  # noqa: E501

        self._project_id = project_id

    @property
    def source(self):
        """Gets the source of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501


        :return: The source of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._source

    @source.setter
    def source(self, source):
        """Sets the source of this ModelJobJoinedModelJobStatusResponseDto.


        :param source: The source of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :type: str
        """

        self._source = source

    @property
    def step(self):
        """Gets the step of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501


        :return: The step of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._step

    @step.setter
    def step(self, step):
        """Sets the step of this ModelJobJoinedModelJobStatusResponseDto.


        :param step: The step of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :type: str
        """
        if step is None:
            raise ValueError("Invalid value for `step`, must not be `None`")  # noqa: E501

        self._step = step

    @property
    def message(self):
        """Gets the message of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501


        :return: The message of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this ModelJobJoinedModelJobStatusResponseDto.


        :param message: The message of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :type: str
        """

        self._message = message

    @property
    def type(self):
        """Gets the type of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501


        :return: The type of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this ModelJobJoinedModelJobStatusResponseDto.


        :param type: The type of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :type: str
        """

        self._type = type

    @property
    def order(self):
        """Gets the order of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501


        :return: The order of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :rtype: float
        """
        return self._order

    @order.setter
    def order(self, order):
        """Sets the order of this ModelJobJoinedModelJobStatusResponseDto.


        :param order: The order of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :type: float
        """

        self._order = order

    @property
    def created_at(self):
        """Gets the created_at of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501


        :return: The created_at of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this ModelJobJoinedModelJobStatusResponseDto.


        :param created_at: The created_at of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501


        :return: The updated_at of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this ModelJobJoinedModelJobStatusResponseDto.


        :param updated_at: The updated_at of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    @property
    def archived_at(self):
        """Gets the archived_at of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501


        :return: The archived_at of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :rtype: datetime
        """
        return self._archived_at

    @archived_at.setter
    def archived_at(self, archived_at):
        """Sets the archived_at of this ModelJobJoinedModelJobStatusResponseDto.


        :param archived_at: The archived_at of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :type: datetime
        """

        self._archived_at = archived_at

    @property
    def last_updated_by(self):
        """Gets the last_updated_by of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501


        :return: The last_updated_by of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._last_updated_by

    @last_updated_by.setter
    def last_updated_by(self, last_updated_by):
        """Sets the last_updated_by of this ModelJobJoinedModelJobStatusResponseDto.


        :param last_updated_by: The last_updated_by of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :type: str
        """

        self._last_updated_by = last_updated_by

    @property
    def created_by(self):
        """Gets the created_by of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501


        :return: The created_by of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :rtype: str
        """
        return self._created_by

    @created_by.setter
    def created_by(self, created_by):
        """Sets the created_by of this ModelJobJoinedModelJobStatusResponseDto.


        :param created_by: The created_by of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :type: str
        """

        self._created_by = created_by

    @property
    def progress(self):
        """Gets the progress of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501


        :return: The progress of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :rtype: float
        """
        return self._progress

    @progress.setter
    def progress(self, progress):
        """Sets the progress of this ModelJobJoinedModelJobStatusResponseDto.


        :param progress: The progress of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :type: float
        """

        self._progress = progress

    @property
    def metadata(self):
        """Gets the metadata of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501


        :return: The metadata of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :rtype: object
        """
        return self._metadata

    @metadata.setter
    def metadata(self, metadata):
        """Sets the metadata of this ModelJobJoinedModelJobStatusResponseDto.


        :param metadata: The metadata of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :type: object
        """

        self._metadata = metadata

    @property
    def version(self):
        """Gets the version of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501


        :return: The version of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :rtype: float
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this ModelJobJoinedModelJobStatusResponseDto.


        :param version: The version of this ModelJobJoinedModelJobStatusResponseDto.  # noqa: E501
        :type: float
        """

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(ModelJobJoinedModelJobStatusResponseDto, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, ModelJobJoinedModelJobStatusResponseDto):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



try:
    import docker
    import pyspark
except ImportError as exc:
    raise ImportError(
        "Some optional dependencies required for curia.infra are missing. "
        "Please install them via `poetry install --with=infra` if developing or"
        " `pip install curia[infra]` if using curia as a library."
    ) from exc



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 3.11.0
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class UpdateFeatureTableDto(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'name': 'str',
        'description': 'str'
    }

    attribute_map = {
        'name': 'name',
        'description': 'description'
    }

    def __init__(self, name=None, description=None):  # noqa: E501
        """UpdateFeatureTableDto - a model defined in Swagger"""  # noqa: E501
        self._name = None
        self._description = None
        self.discriminator = None
        if name is not None:
            self.name = name
        if description is not None:
            self.description = description

    @property
    def name(self):
        """Gets the name of this UpdateFeatureTableDto.  # noqa: E501


        :return: The name of this UpdateFeatureTableDto.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this UpdateFeatureTableDto.


        :param name: The name of this UpdateFeatureTableDto.  # noqa: E501
        :type: str
        """

        self._name = name

    @property
    def description(self):
        """Gets the description of this UpdateFeatureTableDto.  # noqa: E501


        :return: The description of this UpdateFeatureTableDto.  # noqa: E501
        :rtype: str
        """
        return self._description

    @description.setter
    def description(self, description):
        """Sets the description of this UpdateFeatureTableDto.


        :param description: The description of this UpdateFeatureTableDto.  # noqa: E501
        :type: str
        """

        self._description = description

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(UpdateFeatureTableDto, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, UpdateFeatureTableDto):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other



# coding: utf-8

"""
    Curia Platform API

    These are the docs for the curia platform API. To test, generate an authorization token first.  # noqa: E501

    OpenAPI spec version: 0.0.1
    
    Generated by: https://github.com/swagger-api/swagger-codegen.git
"""

import pprint
import re  # noqa: F401

import six

class Environment(object):
    """NOTE: This class is auto generated by the swagger code generator program.

    Do not edit the class manually.
    """
    """
    Attributes:
      swagger_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    swagger_types = {
        'id': 'str',
        'name': 'str',
        'cluster_id': 'str',
        'cluster_status': 'str',
        'config': 'object',
        'type': 'str',
        'terminate_after_idle_minutes': 'float',
        'idle_alarm_name': 'str',
        'concurrency': 'float',
        'organization_id': 'str',
        'model_jobs': 'list[ModelJob]',
        'activity_marker': 'str',
        'activities': 'list[EnvironmentActivity]',
        'created_at': 'datetime',
        'updated_at': 'datetime',
        'version': 'float'
    }

    attribute_map = {
        'id': 'id',
        'name': 'name',
        'cluster_id': 'clusterId',
        'cluster_status': 'clusterStatus',
        'config': 'config',
        'type': 'type',
        'terminate_after_idle_minutes': 'terminateAfterIdleMinutes',
        'idle_alarm_name': 'idleAlarmName',
        'concurrency': 'concurrency',
        'organization_id': 'organizationId',
        'model_jobs': 'modelJobs',
        'activity_marker': 'activityMarker',
        'activities': 'activities',
        'created_at': 'createdAt',
        'updated_at': 'updatedAt',
        'version': 'version'
    }

    def __init__(self, id=None, name=None, cluster_id=None, cluster_status=None, config=None, type=None, terminate_after_idle_minutes=120, idle_alarm_name=None, concurrency=1, organization_id=None, model_jobs=None, activity_marker=None, activities=None, created_at=None, updated_at=None, version=None):  # noqa: E501
        """Environment - a model defined in Swagger"""  # noqa: E501
        self._id = None
        self._name = None
        self._cluster_id = None
        self._cluster_status = None
        self._config = None
        self._type = None
        self._terminate_after_idle_minutes = None
        self._idle_alarm_name = None
        self._concurrency = None
        self._organization_id = None
        self._model_jobs = None
        self._activity_marker = None
        self._activities = None
        self._created_at = None
        self._updated_at = None
        self._version = None
        self.discriminator = None
        if id is not None:
            self.id = id
        self.name = name
        if cluster_id is not None:
            self.cluster_id = cluster_id
        if cluster_status is not None:
            self.cluster_status = cluster_status
        if config is not None:
            self.config = config
        self.type = type
        if terminate_after_idle_minutes is not None:
            self.terminate_after_idle_minutes = terminate_after_idle_minutes
        if idle_alarm_name is not None:
            self.idle_alarm_name = idle_alarm_name
        if concurrency is not None:
            self.concurrency = concurrency
        if organization_id is not None:
            self.organization_id = organization_id
        if model_jobs is not None:
            self.model_jobs = model_jobs
        if activity_marker is not None:
            self.activity_marker = activity_marker
        if activities is not None:
            self.activities = activities
        if created_at is not None:
            self.created_at = created_at
        if updated_at is not None:
            self.updated_at = updated_at
        if version is not None:
            self.version = version

    @property
    def id(self):
        """Gets the id of this Environment.  # noqa: E501


        :return: The id of this Environment.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this Environment.


        :param id: The id of this Environment.  # noqa: E501
        :type: str
        """

        self._id = id

    @property
    def name(self):
        """Gets the name of this Environment.  # noqa: E501


        :return: The name of this Environment.  # noqa: E501
        :rtype: str
        """
        return self._name

    @name.setter
    def name(self, name):
        """Sets the name of this Environment.


        :param name: The name of this Environment.  # noqa: E501
        :type: str
        """
        if name is None:
            raise ValueError("Invalid value for `name`, must not be `None`")  # noqa: E501

        self._name = name

    @property
    def cluster_id(self):
        """Gets the cluster_id of this Environment.  # noqa: E501


        :return: The cluster_id of this Environment.  # noqa: E501
        :rtype: str
        """
        return self._cluster_id

    @cluster_id.setter
    def cluster_id(self, cluster_id):
        """Sets the cluster_id of this Environment.


        :param cluster_id: The cluster_id of this Environment.  # noqa: E501
        :type: str
        """

        self._cluster_id = cluster_id

    @property
    def cluster_status(self):
        """Gets the cluster_status of this Environment.  # noqa: E501


        :return: The cluster_status of this Environment.  # noqa: E501
        :rtype: str
        """
        return self._cluster_status

    @cluster_status.setter
    def cluster_status(self, cluster_status):
        """Sets the cluster_status of this Environment.


        :param cluster_status: The cluster_status of this Environment.  # noqa: E501
        :type: str
        """

        self._cluster_status = cluster_status

    @property
    def config(self):
        """Gets the config of this Environment.  # noqa: E501


        :return: The config of this Environment.  # noqa: E501
        :rtype: object
        """
        return self._config

    @config.setter
    def config(self, config):
        """Sets the config of this Environment.


        :param config: The config of this Environment.  # noqa: E501
        :type: object
        """

        self._config = config

    @property
    def type(self):
        """Gets the type of this Environment.  # noqa: E501


        :return: The type of this Environment.  # noqa: E501
        :rtype: str
        """
        return self._type

    @type.setter
    def type(self, type):
        """Sets the type of this Environment.


        :param type: The type of this Environment.  # noqa: E501
        :type: str
        """
        if type is None:
            raise ValueError("Invalid value for `type`, must not be `None`")  # noqa: E501

        self._type = type

    @property
    def terminate_after_idle_minutes(self):
        """Gets the terminate_after_idle_minutes of this Environment.  # noqa: E501


        :return: The terminate_after_idle_minutes of this Environment.  # noqa: E501
        :rtype: float
        """
        return self._terminate_after_idle_minutes

    @terminate_after_idle_minutes.setter
    def terminate_after_idle_minutes(self, terminate_after_idle_minutes):
        """Sets the terminate_after_idle_minutes of this Environment.


        :param terminate_after_idle_minutes: The terminate_after_idle_minutes of this Environment.  # noqa: E501
        :type: float
        """

        self._terminate_after_idle_minutes = terminate_after_idle_minutes

    @property
    def idle_alarm_name(self):
        """Gets the idle_alarm_name of this Environment.  # noqa: E501


        :return: The idle_alarm_name of this Environment.  # noqa: E501
        :rtype: str
        """
        return self._idle_alarm_name

    @idle_alarm_name.setter
    def idle_alarm_name(self, idle_alarm_name):
        """Sets the idle_alarm_name of this Environment.


        :param idle_alarm_name: The idle_alarm_name of this Environment.  # noqa: E501
        :type: str
        """

        self._idle_alarm_name = idle_alarm_name

    @property
    def concurrency(self):
        """Gets the concurrency of this Environment.  # noqa: E501


        :return: The concurrency of this Environment.  # noqa: E501
        :rtype: float
        """
        return self._concurrency

    @concurrency.setter
    def concurrency(self, concurrency):
        """Sets the concurrency of this Environment.


        :param concurrency: The concurrency of this Environment.  # noqa: E501
        :type: float
        """

        self._concurrency = concurrency

    @property
    def organization_id(self):
        """Gets the organization_id of this Environment.  # noqa: E501


        :return: The organization_id of this Environment.  # noqa: E501
        :rtype: str
        """
        return self._organization_id

    @organization_id.setter
    def organization_id(self, organization_id):
        """Sets the organization_id of this Environment.


        :param organization_id: The organization_id of this Environment.  # noqa: E501
        :type: str
        """

        self._organization_id = organization_id

    @property
    def model_jobs(self):
        """Gets the model_jobs of this Environment.  # noqa: E501


        :return: The model_jobs of this Environment.  # noqa: E501
        :rtype: list[ModelJob]
        """
        return self._model_jobs

    @model_jobs.setter
    def model_jobs(self, model_jobs):
        """Sets the model_jobs of this Environment.


        :param model_jobs: The model_jobs of this Environment.  # noqa: E501
        :type: list[ModelJob]
        """

        self._model_jobs = model_jobs

    @property
    def activity_marker(self):
        """Gets the activity_marker of this Environment.  # noqa: E501


        :return: The activity_marker of this Environment.  # noqa: E501
        :rtype: str
        """
        return self._activity_marker

    @activity_marker.setter
    def activity_marker(self, activity_marker):
        """Sets the activity_marker of this Environment.


        :param activity_marker: The activity_marker of this Environment.  # noqa: E501
        :type: str
        """

        self._activity_marker = activity_marker

    @property
    def activities(self):
        """Gets the activities of this Environment.  # noqa: E501


        :return: The activities of this Environment.  # noqa: E501
        :rtype: list[EnvironmentActivity]
        """
        return self._activities

    @activities.setter
    def activities(self, activities):
        """Sets the activities of this Environment.


        :param activities: The activities of this Environment.  # noqa: E501
        :type: list[EnvironmentActivity]
        """

        self._activities = activities

    @property
    def created_at(self):
        """Gets the created_at of this Environment.  # noqa: E501


        :return: The created_at of this Environment.  # noqa: E501
        :rtype: datetime
        """
        return self._created_at

    @created_at.setter
    def created_at(self, created_at):
        """Sets the created_at of this Environment.


        :param created_at: The created_at of this Environment.  # noqa: E501
        :type: datetime
        """

        self._created_at = created_at

    @property
    def updated_at(self):
        """Gets the updated_at of this Environment.  # noqa: E501


        :return: The updated_at of this Environment.  # noqa: E501
        :rtype: datetime
        """
        return self._updated_at

    @updated_at.setter
    def updated_at(self, updated_at):
        """Sets the updated_at of this Environment.


        :param updated_at: The updated_at of this Environment.  # noqa: E501
        :type: datetime
        """

        self._updated_at = updated_at

    @property
    def version(self):
        """Gets the version of this Environment.  # noqa: E501


        :return: The version of this Environment.  # noqa: E501
        :rtype: float
        """
        return self._version

    @version.setter
    def version(self, version):
        """Sets the version of this Environment.


        :param version: The version of this Environment.  # noqa: E501
        :type: float
        """

        self._version = version

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.swagger_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value
        if issubclass(Environment, dict):
            for key, value in self.items():
                result[key] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, Environment):
            return False

        return self.__dict__ == other.__dict__

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        return not self == other
